package com.nisum.product_consumer.service;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.nisum.product_consumer.entity.Product;
import com.nisum.product_consumer.exception.InvalidMessageException;
import com.nisum.product_consumer.exception.ProductProcessingException;
import com.nisum.product_consumer.repository.ProductConsumerRepository;
import jakarta.annotation.PostConstruct;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Mono;
import reactor.kafka.receiver.KafkaReceiver;
import reactor.kafka.receiver.ReceiverRecord;
import reactor.util.retry.Retry;

import java.time.Duration;


@Service
//@RequiredArgsConstructor
//@Slf4j
public class KafkaConsumerServiceImpl {

    private Logger log = LoggerFactory.getLogger(KafkaConsumerServiceImpl.class);
    private final KafkaReceiver<String, String> kafkaReceiver;
    private final ProductConsumerRepository productRepository;
    private final ObjectMapper objectMapper;

    public KafkaConsumerServiceImpl(KafkaReceiver<String, String> kafkaReceiver, ProductConsumerRepository productRepository, ObjectMapper objectMapper) {
        this.kafkaReceiver = kafkaReceiver;
        this.productRepository = productRepository;
        this.objectMapper = objectMapper;
    }

    @PostConstruct
    public void startConsuming() {
        consumeMessages();
    }

    public void consumeMessages() {
        kafkaReceiver.receive()
                .flatMap(this::processMessage)
                .onErrorContinue((error, obj) -> log.error("Error processing message: {}", error.getMessage(), error))
                .subscribe();
    }


    private Mono<Void> processMessage(ReceiverRecord<String, String> record) {
        return Mono.just(record.value())
                .filter(StringUtils::isNotBlank)
                .switchIfEmpty(Mono.error(new InvalidMessageException("Received blank message from Kafka.")))
                .flatMap(this::parseProduct)
                .flatMap(this::validateProduct)
                .flatMap(productRepository::save)
                .doOnSuccess(product -> log.info("Successfully saved product: {}", product))
                .doOnError(error -> log.error("Error processing product: {}", error.getMessage(), error))
                .onErrorResume(InvalidMessageException.class, error -> {
                    log.warn("Skipping invalid message: {}", error.getMessage());
                    return Mono.empty();
                })
                .onErrorResume(ProductProcessingException.class, error -> {
                    log.error("Product processing failed: {}", error.getMessage());
                    return Mono.empty();
                })
                .retryWhen(Retry.fixedDelay(3, Duration.ofSeconds(2)) // Retry 3 times on failure
                        .filter(error -> error instanceof ProductProcessingException) // Only retry on processing failure
                        .onRetryExhaustedThrow((retryBackoffSpec, signal) -> signal.failure()))
                .then(Mono.fromRunnable(() -> record.receiverOffset().acknowledge())); // Commit offset if successful
    }

    private Mono<Product> parseProduct(String message) {
        try {
            Product product = new ObjectMapper().readValue(message, Product.class);
            return Mono.just(product);
        } catch (Exception e) {
            log.error("Error parsing JSON message: {}", message, e);
            return Mono.empty();
        }
    }

    private Mono<Product> validateProduct(Product product) {
        if (ObjectUtils.isEmpty(product)) {
            log.warn("Product object is null or empty. Skipping processing.");
            return Mono.empty();
        }

        if (StringUtils.isBlank(product.getName()) || ObjectUtils.isEmpty(product.getPrice()) || product.getPrice() <= 0) {
            log.warn("Invalid product data: {}", product);
            return Mono.empty();
        }
        return Mono.just(product);
    }
}
